# Copyright (C) 2010 Linaro Limited
#
# Author: Zygmunt Krynicki <zygmunt.krynicki@linaro.org>
#
# This file is part of django-restricted-resource.
#
# django-restricted-resource is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License version 3
# as published by the Free Software Foundation
#
# django-restricted-resource is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with django-restricted-resource.  If not, see <http://www.gnu.org/licenses/>.

"""
Module with model manager for RestrictedResource
"""

from django.contrib.auth.models import (User, AnonymousUser, Group)
from django.db.models import Q
from django.db import models

from django_restricted_resource.utils import filter_bogus_users


class RestrictedResourceManager(models.Manager):
    """
    Model manager for RestrictedResource and subclasses that has
    additional methods.

    The extra methods are:
        * owned_by_principal(principal)
        * accessible_by_principal(principal)

    Both methods allow for efficient enumeration of owned and accessible
    resources (respectively).

    There is one extra convenience method:
        * accessible_by_anyone()
    That is equivalent to accessible_by_principal(None) but is more
    expressive in intent.
    """

    def owned_by_principal(self, principal):
        """
        Return a QuestySet of RestrictedResource instances that are
        owned by the specified principal, which can be a User or Group
        instance.
        """
        if isinstance(principal, (User, AnonymousUser, type(None))):
            user = filter_bogus_users(principal)
            return self._owned_by_user(user)
        elif isinstance(principal, Group):
            group = principal
            return self._owned_by_group(group)
        else:
            raise TypeError("Expected User or Group instance as argument")

    def accessible_by_principal(self, principal):
        """
        Return a QuerySet of RestrictedResource instances that can be
        accessed by specified principal. The principal may be None,
        AnonymousUser, valid User or valid Group.

        Note: All objects that can be accessed are returned, not just
        objects with exclusive access. To determine why a particular
        principal can access a particular resource use
        RestrictedResource.get_access_type(principal).
        """
        if isinstance(principal, (User, AnonymousUser, type(None))):
            user = filter_bogus_users(principal)
            return self._accessible_by_user(user)
        elif isinstance(principal, Group):
            group = principal
            return self._accessible_by_group(group)
        else:
            raise TypeError("Expected User or Group instance as argument")

    def accessible_by_anyone(self):
        """
        Return a QuerySet of BundleStream instances that can be accessed
        by anyone.
        """
        return self._accessible_by_user(None)

    def _accessible_by_user(self, user):
        if user is None:
            return self.filter(is_public=True)
        else:
            return self.filter(
                Q(is_public=True) |
                Q(user=user) |
                Q(group__in=user.groups.all()))

    def _accessible_by_group(self, group):
        # None gets mapped to users, there is no chance to get None here
        assert group is not None
        return self.filter(
            Q(is_public=True) |
            Q(group=group))

    def _owned_by_user(self, user):
        if user is None:
            return self.none()
        else:
            return self.filter(
                Q(user=user) |
                Q(group__in=user.groups.all()))

    def _owned_by_group(self, group):
        # None gets mapped to users, there is no chance to get None here
        assert group is not None
        return self.filter(group=group)



#!/usr/bin/env python
# Copyright (C) 2010 Linaro Limited
#
# Author: Zygmunt Krynicki <zygmunt.krynicki@linaro.org>
#
# This file is part of django-restricted-resource.
#
# django-restricted-resource is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License version 3
# as published by the Free Software Foundation
#
# django-restricted-resource is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with django-restricted-resource.  If not, see <http://www.gnu.org/licenses/>.

import os
import sys


def find_sources():
    base_path = os.path.join(
        os.path.dirname(os.path.abspath(__file__)), "../..")
    if os.path.exists(os.path.join(base_path, "django_restricted_resource")):
        sys.path.insert(0, base_path)


if __name__ == "__main__":
    find_sources()
    settings_module = "django_restricted_resource.test_project.settings"
    os.environ.setdefault("DJANGO_SETTINGS_MODULE", settings_module)
    from django.core.management import execute_from_command_line
    execute_from_command_line(sys.argv)



# Copyright (C) 2010 Linaro Limited
#
# Author: Zygmunt Krynicki <zygmunt.krynicki@linaro.org>
#
# This file is part of django-restricted-resource.
#
# django-restricted-resource is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License version 3
# as published by the Free Software Foundation
#
# django-restricted-resource is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with django-restricted-resource.  If not, see <http://www.gnu.org/licenses/>.

"""
Unit tests for django-restricted-resources
"""

from django.core.exceptions import ValidationError
from django.contrib.auth.models import (AnonymousUser, User, Group)
from django.db import models

from django_restricted_resource.utils import filter_bogus_users
from django_restricted_resource.managers import RestrictedResourceManager


class RestrictedResource(models.Model):
    """
    Model for handling resource ownership and access rights.

    This is not a generic permission framework. Instead it's a rigid and
    simple but arguably useful model with the following properties:
        * resources are owned by exactly one user or exactly one group
        * resources are accessible to anyone if marked as public
        * resources are accessible to owner if owned by user
        * resources are accessible to members if owned by group
        * What the access actually permits is not defined

    This similar to the UNIX permission system with the following
    differences:
        * There are no separate READ, WRITE, EXECUTE nor STICKY bits per
          any of the possible user classes (OWNER, GROUP MEMBER, OTHER).
          Instead the application is free to use any desired policy to
          define actions a particular user may invoke on a resource that
          he can already access.
        * There is a dedicated "public" flag indicating that anyone can
          access the specified resource.
        * (important) Enumeration of resources that user can access is
          very efficient when implemented in a relational database.
    """
    NO_ACCESS, PUBLIC_ACCESS, PRIVATE_ACCESS, SHARED_ACCESS = range(4)

    user = models.ForeignKey(User, null=True, blank=True)
    group = models.ForeignKey(Group, null=True, blank=True)
    is_public = models.BooleanField(default=False)

    objects = RestrictedResourceManager()

    class Meta:
        abstract = True

    def save(self, *args, **kwargs):
        """
        Validate and save the resource
        """
        self.clean()
        return super(RestrictedResource, self).save(*args, **kwargs)

    def clean(self):
        """
        Make sure that ownership constraint is met.

        RestrictedResource must be owned by exactly one user or exactly
        one group.
        """
        if self.user is not None and self.group is not None:
            raise ValidationError(
                'Cannot be owned by user and group at the same time')
        if self.user is None and self.group is None:
            raise ValidationError(
                'Must be owned by someone')
        if hasattr(models.Model, "clean"):
            return super(RestrictedResource, self).clean()

    def _set_owner(self, owner):
        if not isinstance(owner, (User, Group)):
            raise TypeError("Expected User or Group instance as argument")
        if isinstance(owner, User):
            self.user = owner
            self.group = None
        else:
            self.group = owner
            self.user = None

    def _get_owner(self):
        return self.user or self.group

    owner = property(
        _get_owner, _set_owner, None,
        "Principal owner (either User or Group) of this restricted resource")

    def is_accessible_by(self, principal):
        """
        Check if the resource can be accessed by the specified principal.
        """
        return self.get_access_type(principal) != self.NO_ACCESS

    def is_owned_by(self, principal):
        """
        Check if the resource is owned by the specified principal.

        The principal may be any user or group
        """
        if principal is None:
            return False
        # If the principal is an User then this object is owned by that user or
        # the group the user belongs to.
        if isinstance(principal, (User, AnonymousUser, type(None))):
            user = filter_bogus_users(principal)
            if user is None:
                return False
            if self.user is not None:
                return self.user == user
            else:
                return self.group in user.groups.all()
        # If the principal is a Group then this object is owned by that group
        elif isinstance(principal, Group):
            group = principal
            return self.group == group
        else:
            raise TypeError("Expected User or Group instance as argument")

    def get_access_type(self, principal):
        """
        Determine the mode of access the principal has to this object:

        Possible access types:
            NO_ACCESS
                No access to restricted resource
            PUBLIC_ACCESS
                Granted to:
                    * User (including owner) accessing publicly
                      available resource
                    * Groups accessing publicly available resource
                    * None (including Anonymous users and blocked users)
                      accessing publicly available resources
            PRIVATE_ACCESS
                Granted to:
                    * Owner accessing otherwise private resource
            SHARED_ACCESS
                Granted to:
                    * User accessing restricted data via membership
                    * Groups accessing restricted data they own
        """
        if isinstance(principal, (User, AnonymousUser, type(None))):
            user = filter_bogus_users(principal)
            return self._get_access_type_for_user(user)
        elif isinstance(principal, Group):
            group = principal
            return self._get_access_type_for_group(group)
        else:
            raise TypeError("Expected User or Group instance as argument")

    def _get_access_type_for_user(self, user):
        # Allow anyone to access public data
        if self.is_public:
            return self.PUBLIC_ACCESS
        # Allow access for owners
        if self.user is not None and self.user == user:
            return self.PRIVATE_ACCESS
        # Allow access for team members
        if self.group is not None and user is not None:
            if self.group in user.groups.all():
                return self.SHARED_ACCESS
        # Finally, disallow
        return self.NO_ACCESS

    def _get_access_type_for_group(self, group):
        # Allow anyone to access public data
        if self.is_public:
            return self.PUBLIC_ACCESS
        # Allow access to group if group is the owner
        if self.group is not None and self.group == group:
            return self.SHARED_ACCESS
        # Finally, disallow
        return self.NO_ACCESS



# Copyright (C) 2010 Linaro Limited
#
# Author: Zygmunt Krynicki <zygmunt.krynicki@linaro.org>
#
# This file is part of django-restricted-resource.
#
# django-restricted-resource is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License version 3
# as published by the Free Software Foundation
#
# django-restricted-resource is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with django-restricted-resource.  If not, see <http://www.gnu.org/licenses/>.


def filter_bogus_users(user):
    """
    Check for bogus instances of user (anonymous, disabled, etc) and
    change them to None. This simplifies user comparison and handling.
    If the user is `None' then he's not trusted, period, no need to
    check deeper.
    """
    if user is not None and user.is_authenticated() and user.is_active:
        return user



# Copyright (C) 2010 Linaro Limited
#
# Author: Zygmunt Krynicki <zygmunt.krynicki@linaro.org>
#
# This file is part of django-restricted-resource.
#
# django-restricted-resource is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License version 3
# as published by the Free Software Foundation
#
# django-restricted-resource is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with django-restricted-resource.  If not, see <http://www.gnu.org/licenses/>.

import inspect
import itertools

from django.contrib.auth.models import (AnonymousUser, User, Group)
from django.db import models
from django_testscenarios.ubertest import (TestCase, TestCaseWithScenarios)

from django_restricted_resource.models import RestrictedResource


class TestCaseWithInvariants(TestCaseWithScenarios):
    """
    TestCase that generates test scenarios based on the
    possible combination of invariants.

    Invariants are declared by the 'invariants' variable defined in
    class scope. The variable must point to a dictionary. Each element
    of that dictionary becomes a test invariant. The value of that
    dictionary may be one of:
        * list: Each of the elements is checked. If all elements are of
          simple type (int, bool, string) then parameter description
          will contain them as-is, otherwise they will be described as
          "variant-N"
        * dict: Each of the values is checked. In addition the keys will
          be used to construct meaningful parameter descriptions.
    """

    def _get_invariants(self):
        return getattr(self, 'invariants', {})

    def _get_scenarios(self):
        if hasattr(self, "scenarios"):
            return self.scenarios
        self.scenarios = self._get_all_possible_scenarios(
            self._get_invariants())
        return self.scenarios

    def _dict_to_keys_and_values(self, d):
        pairs = d.items()
        keys = [first for (first, second) in pairs]
        values = [second for (first, second) in pairs]
        return keys, values

    def _get_all_possible_scenarios(self, invariants):
        scenarios = []
        invariant_keys, invariant_values = self._dict_to_keys_and_values(invariants)
        scenario_ids_list = []
        scenario_params_list = []
        for value in invariant_values:
            if isinstance(value, list):
                if all([isinstance(variant, (int, bool, str)) for variant in value]):
                    scenario_ids_list.append([repr(variant) for variant in value])
                else:
                    scenario_ids_list.append(['variant-%d' % variant for variant in range(len(value))])
                scenario_params_list.append(value)
            elif isinstance(value, dict):
                k, v = self._dict_to_keys_and_values(value)
                scenario_ids_list.append(k)
                scenario_params_list.append(v)
        for scenario_ids, scenario_params in itertools.izip(
            itertools.product(*scenario_ids_list),
                itertools.product(*scenario_params_list)):
            parameters = dict(zip(invariant_keys, scenario_params))
            name = ", ".join([
                "%s=%s" % (invariant_key, param_id)
                for (invariant_key, param_id)
                in zip(invariant_keys, scenario_ids)])
            scenario = (name, parameters)
            scenarios.append(scenario)
        return scenarios

    def setUp(self):
        super(TestCaseWithScenarios, self).setUp()
        # Evaluate lazy invariants now that `self' is around
        for invariant in self._get_invariants().iterkeys():
            value = getattr(self, invariant)
            if inspect.isfunction(value):
                value = value(self)
                setattr(self, invariant, value)


class ExampleRestrictedResource(RestrictedResource):
    """
    Dummy model to get non-abstract model that inherits from
    RestrictedResource
    """
    name = models.CharField(max_length=100, null=True, unique=True)

    class Meta:
        ordering = ['name']


class FixtureHelper(object):

    def getUniqueString(self, prefix=None, max_length=None):
        value = super(FixtureHelper, self).getUniqueString(prefix)
        if max_length is not None:
            if len(value) >= max_length:
                value = super(FixtureHelper, self).getUniqueString("short")
                if len(value) >= max_length:
                    raise ValueError("Unable to satisfy request for random string with max_length=%d" % max_length)
        return value

    def getUniqueStringForField(self, model, field_name):
        return self.getUniqueString(max_length=model._meta.get_field_by_name(field_name)[0].max_length)

    def getUniqueUser(self, is_active=True):
        user = User.objects.create(
            username=self.getUniqueStringForField(User, "username"),
            is_active=is_active)
        self.addCleanup(user.delete)
        return user

    def getUniqueGroup(self):
        group = Group.objects.create(
            name=self.getUniqueStringForField(Group, "name"))
        self.addCleanup(group.delete)
        return group

    def getUniqueResource(self, owner, is_public, name=None):
        resource = ExampleRestrictedResource.objects.create(
            owner=owner, is_public=is_public, name=name)
        self.addCleanup(resource.delete)
        return resource

    def add_resources(self, resources, owner, is_public):
        for name in resources:
            self.getUniqueResource(
                name=name, owner=owner, is_public=is_public)



#!/usr/bin/env python
# Copyright (C) 2010 Linaro Limited
#
# Author: Zygmunt Krynicki <zygmunt.krynicki@linaro.org>
#
# This file is part of django-restricted-resource.
#
# django-restricted-resource is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License version 3
# as published by the Free Software Foundation
#
# django-restricted-resource is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with django-restricted-resource.  If not, see <http://www.gnu.org/licenses/>.

from setuptools import setup


setup(
    name='django-restricted-resource',
    version="2014.04",
    author="Linaro Limited",
    author_email="lava-team@linaro.org",
    description="Base model for Django that adds simple and efficient ownership and access control.",
    url='http://git.linaro.org/git/lava/django-restricted-resource.git',
    test_suite='django_restricted_resource.test_project.tests.run_tests',
    license='LGPLv3',
    keywords=['django', 'ownership', 'models'],
    classifiers=[
        "Development Status :: 4 - Beta",
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        "License :: OSI Approved :: GNU Library or Lesser General Public License (LGPL)",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 2.6",
        "Programming Language :: Python :: 2.7",
    ],
    zip_safe=True,
    packages=['django_restricted_resource'],
    # dependencies
    install_requires=['django >= 1.6'],
    tests_require=['django-testscenarios >= 0.7.1'],
)
